# PostCSS Filter Rules

[![npm version](https://badge.fury.io/js/postcss-filter-rules.svg)](https://badge.fury.io/js/postcss-filter-rules) [![License](https://img.shields.io/npm/l/postcss-filter-rules.svg)](https://github.com/lachlanmcdonald/postcss-filter-rules/blob/master/LICENSE) [![Build](https://github.com/lachlanmcdonald/postcss-filter-rules/actions/workflows/build.yml/badge.svg?branch=master)](https://github.com/lachlanmcdonald/postcss-filter-rules/actions/)

[PostCSS] plugin that filters rules with a callback function on each selector and at-rule. For instance, to filter out individual rules or remove all rules besides those you wish to keep.

## Installation

```shell
npm install --save-dev postcss postcss-filter-rules
```

## Usage

```js
postcss([
    require('postcss-filter-rules')(options)
])
```

See [PostCSS] docs for examples for your environment.

## Examples

### Filtering selectors

To keep only the selectors which contain the `.styleguide` class in the following CSS:

```css
.styleguide span,
.button span {
	color: red;
}
.button {
	color: blue;
}
```

Use the following `filter`:

```js
{
	filter: (selector, parts) => parts.includes('.styleguide')
}
```

Which will output:

```css
.styleguide span {
	color: red;
}
```

## Options

__postcss-filter-rules__ accepts options with following signature:

```ts
{
    filter(selector: string, parts: string[]): boolean,
    splitFunction(selector: string): string[],
}
```

__Note:__ `keepAtRules` is deprecated as of 0.8.0 and will raise a warning. As such, all [at-rules] are retained by default unless empty. Plugins, such as [postcss-discard-unused](https://www.npmjs.com/package/postcss-discard-unused), are better fit-for-purpose for removing at-rules which are no longer referenced in the resulting CSS, such as `@keyframes` or `@font-face`.

### filter

Called for each selector in a rule:

- If the function returns a`true`, the selector is retained. Otherwise, the selector is removed.
- If all of the selectors for a rule are removed, the rule is also removed.
- If all rules within an [at-rule] are removed, the rule is also removed.

The function receives two arguments:

- **selector** (`string`): The selector, as authored in CSS
- **parts** (`string[]`): An array of elements, classes, IDs and pseudo-classes in the selector. Used to quickly search for the existence of classes or IDs.

### splitFunction

A custom callback that can be used to generate the `parts` argument of the callback. If one is not provided, the [default callback](https://github.com/lachlanmcdonald/postcss-filter-rules/blob/master/splitSelectors.js) is used.

[PostCSS]:       https://github.com/postcss/postcss
[at-rule]:       https://developer.mozilla.org/en-US/docs/Web/CSS/At-rule
[at-rules]:      https://developer.mozilla.org/en-US/docs/Web/CSS/At-rule
