const splitSelectors = require('./splitSelectors');

const plugin = (options = {}) => {
	options = {
		filter: () => true,
		splitFunction: splitSelectors,
		...options,
	};

	return {
		postcssPlugin: 'postcss-filter-rules',
		Once(_root, { result }) {
			// Warn if the "keepAtRules" is used.
			if (Object.hasOwnProperty.call(options, 'keepAtRules')) {
				result.warn('The "keepAtRules" option is deprecated and no longer has any effect.');
			}
		},
		Rule(rule) {
			const selectors = rule.selectors.slice().filter(selector => {
				return options.filter(selector, options.splitFunction(selector), rule) === true;
			});

			if (selectors.length === 0) {
				rule.remove();
			} else {
				rule.selectors = selectors;
			}
		},
		AtRule(rule) {
			// Remove empty rules to preserve behaviour with earlier versions.
			if (['media', 'font-face', 'page', 'supports', 'counter-style', 'font-feature-values', 'property', 'layer'].includes(rule.name) && rule.nodes.length === 0) {
				rule.remove();
			}
		},
	};
};

plugin.postcss = true;

module.exports = plugin;
